#include <vector>
#include <list>

#include "graph.h"

Vertex::Vertex(size_t id, string name)
: id(id), name(name) {};

Vertex::Vertex(size_t id, string name, size_t degree)
: id(id), name(name), deg(degree) {};

Vertex::~Vertex() {};

Edge::Edge(Graph &G, size_t id, Vertex *a, Vertex *b)
: id(id), covered(false) {
	end[0] = a;
	end[1] = b;

	a->edges.emplace_back( make_pair(b, this) );
	b->edges.emplace_back( make_pair(a, this) );
	pos[0] = a->edges.size() - 1;
	pos[1] = b->edges.size() - 1;

	// add the edge to G.E
	G.E.push_back(this);
	iterE = G.E.size() - 1;

	// update degrees
	vert_change_deg(G, a, a->deg + 1);
	vert_change_deg(G, b, b->deg + 1);
}


// swap the position of the two end verticies
// just a convenience function as edges are undirected
void Edge::flip() {
	swap(end[0], end[1]);
	swap(pos[0], pos[1]);
}

void Graph::print() {
	cout << "# Printing Graph" << endl;
	for (Vertex *v: this->V) {
		for (auto it = v->edges.begin(); it != v->edges.end(); it++) {
			Vertex *b = it->first;
			if (v->id > b->id)
				continue;

			cout << "# " << v->name << " " << b->name << endl;
		}
	}
	cout << "# Printing done" << endl;
}
