#include <cassert>
#include <cstdlib>
#include <fstream>

#include "graph.h"
#include "connectivity.h"
#include "cut_bound.h"

static void pipe_graph(Graph &G, ofstream &file) {
	for (Vertex *v: G.V) {
		for (auto edge: v->edges) {
			Vertex *u = edge.first;
			if (v->id > u->id)
				continue;

			for (long i = 0; i < v->weight; i++) {
				for (long j = 0; j < u->weight; j++) {
					file << i << "_" << v->name << "\t" << j << "_" << u->name << endl;
				}
			}
		}
	}
}

long long maximum_2club(Graph &G) {
	ofstream pipe_out;
	ifstream pipe_in;
	pipe_out.open("pipe.dimacs", std::ofstream::out | std::ofstream::trunc);

	pipe_graph(G, pipe_out);
	assert(system("./2club_wrapper.sh") == 0);
	pipe_in.open("pipe.solution");
	string output;
	getline(pipe_in, output);

	/* parse the output for the result */
	long long result = 0;
	bool found = false;



	for (size_t i = 0; i < output.size(); i++) {
		string match = "2-club:\t";

		size_t j = i;
		for (; j < output.size(); j++) {
			if (match[j-i] != output[j])
				goto fail;

			if (match[j-i] == '\t') {
				j++;
				break;
			}
		}
		if (j == output.size())
			goto fail;

		for (; j < output.size(); j++) {
			char c = output[j];
			if ('0' <= c && c <= '9') {
				found = true;
				result *= 10;
				result += c - '0';
			}
			else {
				break;
			}
		}
		if (found)
			break;
	fail:
		;
	}

	assert(found);
	return result;
}

long long cut_bound(Graph &G) {
	//WARNING: G must be connected
	return 0;
	
	long long weight = 0;
	for (Vertex *v: G.V)
		weight += v->weight;

	long long two_club = maximum_2club(G);
	long long connectivity = vertex_connectivity(G);
	assert(connectivity != 0);

	return min(connectivity, weight - two_club);
}
