#include <vector>
#include <list>
#include <utility>
#include <cassert>
#include <string>
#include <unordered_map>

#include "graph.h"

/* residual flow graph */

class F_Vertex;
class F_Edge;
class F_Graph;

// do not create these objects yourself, rather use the create functions declared at the bottom
// we use object pooling to increase perfomance significantly

class F_Vertex {
public:
	vector< pair<F_Vertex *, F_Edge *> > edges;
	bool marked = false;
//	string name;
	F_Edge *prev = nullptr;

	F_Vertex();
};

class F_Edge {
public:
	// directed edge
	F_Vertex *a, *b;
	F_Edge *rev;	// residual
	
	Edge *dual = nullptr;

	long cap = 0;
	void send_flow(long flow);

	F_Edge();
};


class F_Graph {
public:
	vector<F_Vertex *> V;
	vector<F_Edge *> E;

	~F_Graph();
	void clear();
};

long compute_max_flow(F_Graph &G, F_Vertex *s, F_Vertex *t);

F_Vertex *create_vertex(F_Graph &G);
F_Edge *create_edge(F_Graph &G, F_Vertex *a, F_Vertex *b, long cap, Edge *dual);
