#pragma once

#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <list>
#include <queue>

using namespace std;

class Edge;
class Vertex;
class F_Vertex; // defined in flow.h
class Graph;

#include "heap.h"
#include "snapshot.h"
#include "constraints.h"

class Vertex {
public:
	size_t id;
	string name;

	// here an edge is a pair of the other vertex and the edge object
	// edges is the list of not covered edges
	// covered edges are stored in the covered list instead
	vector< pair<Vertex *, Edge *> > edges;
	vector< pair<Vertex *, Edge *> > covered;

	// here the degree is the number of uncovered edges that are adjacent
	// meaning deg == edges.size()
	size_t deg = 0;
	long long weight = 1;

	// if its's a degree > 0 vertex it will be in the list V. This is the iterator to it
	size_t iterV;

	// stuff for branching
	size_t component = 0;
	
	// stuff used for read_sol
	list<Vertex *>::iterator iter_sol;
	bool in_sol = false;


	// temporary bool, make sure you reset this back to false if you use it
	bool marked = false;
	bool marked2 = false;
	bool marked3 = false;
	bool visited = false;
	Vertex *tmp = nullptr;


	// stuff for twin
	Vertex *merge = nullptr; // the vertex that we got merged into

	// stuff for disjoint_bound and kplus_disjoint
	int dist = 0;
	long long overlap; // number of remaining P4's this vertex can be part of to compute the lower bound

	long long score = 0;

	// stuff for kplus_disjoint
	vector<F_Vertex *> fvert;

	// stuff for checking constraints
	bool cstr_in_sol = false;	
	bool permanent = false;

	Vertex(size_t id, string name);
	Vertex(size_t id, string name, size_t degree);

	bool contains_edge(Vertex *to);
	virtual ~Vertex();
};

class Edge {
public:
	size_t id;
	// two endpoint vertices
	Vertex *end[2];
	bool covered;

	// iterator storing the position of this edge in G.E
	size_t iterE;

	// the position of this edge in the edgelists of the two endpoint vertices	
	size_t pos[2];

	// swap the position of the two end verticies
	// just a convenience function as edges are undirected
	void flip();

	Edge(Graph &G, size_t id, Vertex *a, Vertex *b);
};

class TWOCVDS {
public:
	// 2-Club Vertex Deletion Set
	vector<Vertex *> S;
	vector<Vertex *> S_backup;
	long long cost = 0;

	/* the edges covered by S */
	vector<Edge *> E;
};

class Graph {
public:
	// V does not contain degree 0 vertices
	vector<Vertex *> V;
	vector<Edge *> E;


	size_t n = 0; // the next vertex id
	size_t m = 0; // the next edge id

	map<string, Vertex *> name_map;

	size_t recursive_steps = 0;
	size_t cuts = 0;

	// the 2-club vertex deletion set
	TWOCVDS VDS;
	vector<GraphModification *> changes;

	// stuff for constraints
	vector<Constraint *> constraints;
	vector<Vertex *> permanent;

	void print();
	bool contains_edge(Vertex *a, Vertex *b);
};

// a functor that compares the degrees of two vertices
struct vertex_cmp {
	bool operator() (const Vertex *lhs, const Vertex *rhs) const {
		if (lhs->deg == rhs->deg)
			return lhs->id < rhs->id;
		return lhs->deg < rhs->deg;
	}
};
