#include <iostream>

#include <string>
#include <sstream>
#include <cassert>
#include <unordered_set>

#include "graph.h"
#include "branch.h"
#include "config.h"
#include "time.h"

using namespace std;



/*********************************************************************************************************************/

static vector<string> split(const string& str, const string& delim)
{
	vector<string> tokens;
	size_t prev = 0, pos = 0;
	do
	{
		pos = str.find(delim, prev);
		if (pos == string::npos) pos = str.length();
		string token = str.substr(prev, pos-prev);
		if (!token.empty()) tokens.push_back(token);
		prev = pos + delim.length();
	}
	while (pos < str.length() && prev < str.length());
	return tokens;
}

int main(int argc, char **argv) {
	// start measuring time
	//TIME_start = chrono::steady_clock::now(); 

	Graph G;

	string line;


	// read configuration from file
	if (argc == 2) {
		read_config_from_file(argv[1]);
	}
	if (argc > 2) {
		cout << "Usage: " << argv[0] << " [config.conf]" << endl;
	}

	// little performance boost for iostream
	std::ios::sync_with_stdio(false);


	// parse the input
	while (getline(cin, line)) {
		if (line[0] == '#' || line.size() == 0)
			continue;

		// for some reason "\r\n" is not handled correctly
		if (!line.empty() && line[line.size() - 1] == '\r')
		    line.erase(line.size() - 1);

		vector<string> vertex = split(line, " ");

		if (vertex.size() != 2) {
			cout << "Parsing error\n";
			return 1;
		}

		// the two verticies of the edge
		Vertex *a, *b;
		a = nullptr;
		b = nullptr;

		// find them
		for (int i = 0; i < 2; i++) {
			string name = vertex[i];

			// create the verticies if they don't already exist
			if (G.name_map.find(name) == G.name_map.end()) {
				Vertex *v = new Vertex(G.n, name);
				G.n++;

				// map the name of the vertex to an id
				G.name_map[name] = v;

				if (i == 0)
					a = v;
				else	
					b = v;
			}
			else {
				if (i == 0)
					a = G.name_map[name];
				else	
					b = G.name_map[name];
			}
		}

		// create the edge
		Edge *e = new Edge(G, G.m, a, b);
		G.m++;
	}

	cout << "# nodes: " << G.V.size() << endl;
	cout << "# edges: " << G.E.size() << endl;

	srand(G.V.size());

	two_club_vertex_delete(G);

	return 0;
}
