#include <cassert>
#include <algorithm>

#include "graph.h"
#include "optimize.h"
#include "snapshot.h"

#include "twin.h"
#include "kplus_disjoint.h"
#include "permanent_club.h"
#include "simple.h"


optimization_rules enabled_rules[NUM_RULES] = {
	OPT_KPLUS_DISJOINT,
	OPT_TWIN,
	OPT_PERMANENT_CLUB,
	OPT_SIMPLE_CHOICE,
	OPT_2CLUB_PURGE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE,
	OPT_NONE
};



/* optimize Graph */
void graph_optimize(Graph &G, long long &k) {
	for (int i = 0; i < NUM_RULES; i++) {
		if (k <= 0)
			break;

		switch (enabled_rules[i]) {
		case OPT_TWIN:
			tc_twin(G);
			break;
		case OPT_KPLUS_DISJOINT:
			tc_kplus_disjoint(G, k);
			break;
		case OPT_PERMANENT_CLUB:
			tc_permanent_club(G, k);
			break;
		case OPT_SIMPLE_CHOICE:
			tc_simple_choice(G, k);
			break;
		case OPT_2CLUB_PURGE:
			tc_2club_purge(G);
			break;
		default:
			break;
		}
	}
}
