#include <vector>
#include <cassert>

#include "graph.h"
#include "read_sol.h"
#include "snapshot.h"
#include "twin.h"

bool tc_twin_single(Graph &G, Vertex *v) {
	// find all twin candidates
	vector<Vertex *> candidates;
	vector<Vertex *> tmp;

	assert(!v->marked);
	v->marked = true;

	for (auto edge: v->edges) {
		Vertex *a = edge.first;
		assert(!a->marked);
		a->marked = true;
		candidates.push_back(a);
	}
	for (Vertex *a: candidates) {
		for (auto edge: a->edges) {
			Vertex *b = edge.first;
			if (!b->marked) {
				tmp.push_back(b);
				b->marked = true;
			}
		}
	}
	candidates.insert(candidates.end(), tmp.begin(), tmp.end());
	for (Vertex *a: candidates) {
		assert(a->marked);
		a->marked = false;
	}

	// mark all neighbours
	for (auto edge: v->edges) {
		Vertex *a = edge.first;
		assert(!a->marked);
		a->marked = true;
	}

	
	
	bool success = false;

	for (Vertex *a: candidates) {
		bool found = true;

		if (a->edges.size() != v->edges.size())
			continue;

		// check if a is a twin
		for (auto edge2: a->edges) {
			Vertex *b = edge2.first;
			if (!b->marked) {
				found = false;
				break;
			}
		}
		if (!found)
			continue;
		
		GM_Twin_Merge *op = new GM_Twin_Merge();
		op->twin = v;
		op->merged = a;

		a->marked = false;
		a->merge = v;
		delete_vertex(G, a);
		v->weight += a->weight;
		G.changes.push_back(op);
		success = true;
		if (a->permanent)
			mark_permanent(G, v);
	}

	v->marked = false;
	for (auto edge: v->edges) {
		Vertex *a = edge.first;
		a->marked = false;
	}

	return success;
}

bool tc_twin(Graph &G) {
	bool success = false;
	vector<Vertex *> V_tmp(G.V);

	for (Vertex *v: V_tmp) {
		if (v->deg == 0)
			continue;

		success = tc_twin_single(G, v) ? true : success;
	}

	return success;
}

void GM_Twin_Merge::undo(Graph &G) {
	twin->weight -= merged->weight;
	merged->merge = nullptr;
}

void GM_Twin_Merge::translate(Graph &G, list<Vertex *> &sol) {
	if (twin->in_sol)
		rs_add_vertex(merged, sol);
}
